#include <lektor/net.h>

/* Private definition of the lkt_message struct */
struct lkt_message {
    size_t ref_count;
    char data[LKT_MESSAGE_MAX];
    size_t data_len;
};

struct lkt_message *
lkt_message_new(void)
{
    struct lkt_message *res = LKT_ALLOC_STRUCT(lkt_message);
    res->ref_count          = 1;
    *(res->data)            = 0;
    res->data_len           = 0;
    return res;
}

struct lkt_message *
lkt_message_new_fmt(const char *restrict fmt, ...)
{
    struct lkt_message *ret = lkt_message_new();
    va_list ap;
    va_start(ap, fmt);
    ret->data_len = (size_t)safe_vsnprintf(ret->data, LKT_MESSAGE_MAX, fmt, ap);
    va_end(ap);
    return ret;
}

struct lkt_message *
lkt_message_new_str(const char *restrict msg)
{
    struct lkt_message *ret = lkt_message_new();
    ret->data_len           = strlen(msg);
    strncpy(ret->data, msg, ret->data_len + 1);
    return ret;
}

void
lkt_message_free(struct lkt_message *const msg)
{
    if ((msg != NULL) && (msg->ref_count > 0)) {
        msg->ref_count--;
        if (msg->ref_count == 0)
            free(msg);
    }
}

struct lkt_message *
lkt_message_clone(struct lkt_message *const msg)
{
    msg->ref_count++;
    return msg;
}

int
lkt_message_send(int fd, const struct lkt_message *const msg)
{
    return (int)send(fd, msg->data, msg->data_len, MSG_NOSIGNAL);
}
